/*
 *  $Id: reduce.c,v 1.5 1993/11/24 00:49:13 doss Exp $
 *
 *  (C) 1993 by Argonne National Laboratory and Mississipi State University.
 *      All rights reserved.  See COPYRIGHT in top-level directory.
 */


#ifndef lint
static char vcid[] = "$Id: reduce.c,v 1.5 1993/11/24 00:49:13 doss Exp $";
#endif /* lint */

#include <mpi.h>
#include "coll.h"

/*@

MPI_Reduce - 

@*/
MPI_Reduce ( sendbuf, recvbuf, count, datatype, op, root, comm )
void             *sendbuf;
void             *recvbuf;
int               count;
MPI_Datatype      datatype;
void            (*op)();
int               root;
MPI_Comm          comm;
{
  MPI_Status status;
  int        size, rank;
  int        errno = MPI_SUCCESS;

  /* Check for invalid arguments */
  if ( (comm            == (MPI_Comm)0) ||
       (root            <  0)           || 
       (comm->comm_type == MPIR_INTER) ) 
    return (errno); 
  
  /* Is root within the communicator? */
  MPI_Comm_size ( comm, &size );
  if ( root >= size )
    return (errno);

  /* Get my rank and switch communicators to the hidden collective */
  MPI_Comm_rank ( comm, &rank );
  comm = comm->comm_coll;

  /* If I'm the root, then I will receive and reduce the messages */
  if (rank == root) {
    int              extent, i;
    void            *buffer;

    /* Copy my buffer to the recvbuf */
    MPI_Type_extent ( datatype, &extent );
    buffer = (void *)MALLOC(extent * count);
    if (sendbuf != recvbuf) {
      MPI_Sendrecv( sendbuf, count, datatype, root, 
		    recvbuf, count, datatype, root, 
		    MPIR_REDUCE_TAG, comm, &status);
    }

    
    /* Receive messages from everywhere and combine buffers */
    for (i=0; i<root; i++) {
      MPI_Recv( buffer, count, datatype, i, MPIR_REDUCE_TAG, comm, &status);
      (*op)(buffer, recvbuf, count, datatype); 
    }
    for (i=root+1; i<size; i++) {
      MPI_Recv( buffer, count, datatype, i, MPIR_REDUCE_TAG, comm, &status);
      (*op)(buffer, recvbuf, count, datatype); 
    }
  }
  else
    MPI_Send( sendbuf, count, datatype, root, MPIR_REDUCE_TAG, comm );

  return (errno);
}


