/*
 *  $Id: scatterv.c,v 1.5 1993/11/24 00:49:13 doss Exp $
 *
 *  (C) 1993 by Argonne National Laboratory and Mississipi State University.
 *      All rights reserved.  See COPYRIGHT in top-level directory.
 */

#ifndef lint
static char vcid[] = "$Id: scatterv.c,v 1.5 1993/11/24 00:49:13 doss Exp $";
#endif /* lint */

#include <mpi.h>
#include "coll.h"

/*@

MPI_Scatterv - 

@*/
int MPI_Scatterv ( sendbuf, displs, sendcnts, sendtype, 
                   recvbuf, recvcnt, recvtype, 
                   root, comm )
void             *sendbuf;
int              *displs;
int              *sendcnts;
MPI_Datatype      sendtype;
void             *recvbuf;
int               recvcnt;
MPI_Datatype      recvtype;
int               root;
MPI_Comm          comm;
{
  MPI_Status status;
  int        rank, size;
  int        errno = MPI_SUCCESS;

  /* Get size and rank */
  MPI_Comm_size ( comm, &size );
  MPI_Comm_rank ( comm, &rank );

  /* Check for invalid arguments */
  if ( (comm            == (MPI_Comm)0) ||
       (root            < 0)            || 
       (root            >= size)        ||
       (recvcnt         <= 0)           ||
       (recvbuf         == (void *)0)   ||
       (comm->comm_type == MPIR_INTER) ) 
    return (errno); 
  
  /* Switch communicators to the hidden collective */
  comm = comm->comm_coll;

  /* If I'm the root, then scatter */
  if ( rank == root ) {
    int i, extent;

    MPI_Type_extent(sendtype, &extent);
    for ( i=0; i<size; i++ )
      MPI_Send( (void *)((int)sendbuf+displs[i]*extent), sendcnts[i], 
		sendtype, i, MPIR_SCATTERV_TAG, comm);
  }

  /* Everybody receives from the root */
  MPI_Recv(recvbuf, recvcnt, recvtype, root, MPIR_SCATTERV_TAG, comm, &status);

  return (errno);
}


